
package macrolanguage;

import java.awt.*;
import java.awt.event.*;
import macrolanguage.*;

/**
 * the class EditToolbar supplies a toolbar for adding and manipulateing
 * objects on the FreeFormEdit.
 * EditToolbar is simply a user interface class which delegates all of
 * the functionality to the free form editing.
**/
public class EditToolbar extends Frame implements ActionListener
{
	// used to load new instances of components automaticaly.
	private MacroExtention loader = new MacroExtention();

	// the owner FreeFormEdit.
	private FreeFormEdit form;
	
	// the entry field in which you select the name of the class
	// to add to the free form edit.
	private TextField classNameEntry = new TextField("java.awt.Button");

	// the entry field in which you select the name of the JAR file
	// in which to search for the class files.
	private TextField jarFileName = new TextField(20);

	// add the class whos name appears in the classNameEntry field.
	private Button addClassButton = new Button("Add class");

	// delete the selected Component.
	private Button deleteSelectedButton = new Button("Delete selected");

	// exit the application.
	private Button exitButton = new Button("Exit");

	/**
	 * The constructor sets the UI to the correct appearance.
	**/
	public EditToolbar(FreeFormEdit form)
	{
		super("Toolbar");

		// the FreeFormEdit instance owning this toolbar.
		this.form = form;

		// the default layout currently. This is subject to
		// profound change.
		setLayout(new FlowLayout());
		
		// adding the components to the toolbar.
		add(classNameEntry);
		add(jarFileName);
		add(addClassButton);
		add(deleteSelectedButton);
		add(exitButton);

		// listening to the component events.
		addClassButton.addActionListener(this);
		deleteSelectedButton.addActionListener(this);
		exitButton.addActionListener(this);
		
		// set the size of the window to fit the Components in it.
		pack();

		// set the FreeFormEdit to be right under this window.
		form.setLocation(getLocation().x , getLocation().y + getSize().height);

		// show the window.
		show();
	}
	public void actionPerformed(java.awt.event.ActionEvent ev)
	{
		// if the aa button is pressed.
		if(ev.getSource() == addClassButton)
		{
			// if a JAR file is selected to be used then draw the class
			// from it.
			if((jarFileName.getText() != null) && (!jarFileName.getText().equals("")))
				loader.setJARFileName(jarFileName.getText());
			
			// find the class coresponding to the string in
			// the text entry and create an instance of it.
			Object temp = loader.loadObject(classNameEntry.getText());

			// if the instance is not invalid then enter add
			// a new component mode with this Component.
			if(temp == null)
			{
				System.out.println("Class loading error.");
				return;
			}
			if (temp instanceof Component)
			{
				form.placeComponentOnNextClick((Component)temp);
			}
			else
			{
				System.out.println("You can only place subclasses of the class Component on the freeform edit.");
			}
		}

		// if the delete button is pressed then delete the 
		// selected Component.
		if(ev.getSource() == deleteSelectedButton)
		{
			form.removeSelection();
		}

		// if the exit button is pressed then ...
		if(ev.getSource() == exitButton)
		{
			System.exit(0);
		}
	}
}