
package macrolanguage;

import java.awt.*;
import java.awt.event.*;
import macrolanguage.EditToolbar;
import macrolanguage.NotAContainerException;
import macrolanguage.ComponentWrapper;

/**
 * the FreeFormEdit class is a class which contains Components on a Frame
 * surface which allows dragging & droping and possible manipulation of 
 * components.
 * This class is without actual inteligence and only exposes functionality
 * to be used by other classes. The internal componets are all wrapped with
 * instances of the class Component wrapper to allow easier editing.
**/
public class FreeFormEdit extends Frame
	  implements  KeyListener, WindowListener
{
   // the currently selected component. By default this frame.
   private Component selection = this;

   // the MouseManager takes over the full responsibility of manageing the
   // mouse events for the form designing.
   private MouseManager mouseManager = new MouseManager(this);

   /**
	* The constructor creates the listeners with the correct layout
	* for the free form design.
   **/
   public FreeFormEdit()
   { 
	  super();

	  addWindowListener(this);
	  addKeyListener(this);
	  addMouseListener(mouseManager);
	  addMouseMotionListener(mouseManager);

	  // set the layout to be a null which prevents the awt from
	  // toying with the layout of the components.
	  setLayout(null);

	  // default size.
	  setSize(300,300);

	  ProjectSettings.currentFreeFormEdit = this;

	  show();
   }         
   /**
	* add a component to an x and y location.
	* This method will be called by the user putting a component on
	* the form or an embeded conainer. This method throws a NotAContainerException
	* when the x and y location points at a component, since components
	* cannot be placed on other components in Java (this is untrue with
	* regarding lightweight components).
   **/
   public void add(Component c, int x, int y) throws NotAContainerException
   {
	  // check whether there is a component at the x and y location
	  // and if so throw NotAContainerException.
	  Component temp = getComponentAt(x,y);
	  if(!(temp instanceof Container))
		 throw (new NotAContainerException("Tried to add a component to something other than a container."));


	  // add the component to the container.
	  ((Container)temp).add(c);

	  // set the relative x and y location.
	  c.setLocation(x, y);
   }      
   /**
	 * this method is a part of the keylistener interface used
	 * to listen to keyboard events.
   **/
   public void keyPressed(java.awt.event.KeyEvent ev)
   {
   }      
   /**
	* this method is a part of the keylistener interface used
	* to listen to keyboard events.
   **/
   public void keyReleased(java.awt.event.KeyEvent ev)
   {
   }   
   /**
	* this method is a part of the keylistener interface used
	* to listen to keyboard events.
   **/
   public void keyTyped(java.awt.event.KeyEvent ev)
   {
   }   
   /**
	* places the component comp on the next click on the free form
	* edit surface.
   **/
   public void placeComponentOnNextClick(Component comp)
   {
	  // change the cursor so it will be evident we are in the middle
	  // of a drop operation.
	  setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
	  mouseManager.placeComponentOnNextClick(comp);
   }   
   /**
	 * removes the currently selected object from the free form edit.
   **/
	public void removeSelection()
	{
		// if the FreeFormEdit is not the currently selected Component
		// the remove it from the surface.
		if(selection != this)
		{
			remove(selection);
			selection = this;
			repaint();
		}
	}   
   /**
	 * set the selection Component to be the selected Component.
	 * This method is used to mark a component as selected.
   **/
   public void setSelection(Component selection)
   {
		if (this.selection != selection)
		{
			if(this.selection != this)
				((ComponentWrapper)this.selection).deselect();
	
			// if selection is an instance of a component wrapper		
			if(selection instanceof ComponentWrapper)
			{
				this.selection = selection;
				// activate the ComponentWrapper's select method.
				((ComponentWrapper)selection).select();
			}
			else
			{
				// find the parent of this component which must be a 
				// Component wrapper.
				if(selection.getParent() instanceof ComponentWrapper)
					setSelection(selection.getParent());
				else
					this.selection = this;
			}
		}
	}   
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
   **/
   public void windowActivated(java.awt.event.WindowEvent ev)
   {
   }   
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
   **/
   public void windowClosed(java.awt.event.WindowEvent ev)
   {
   }   
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
	* this event quit's the application
   **/
   public void windowClosing(java.awt.event.WindowEvent ev)
   {
	  System.exit(0);
   }      
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
   **/
   public void windowDeactivated(java.awt.event.WindowEvent ev)
   {
   }   
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
   **/
   public void windowDeiconified(java.awt.event.WindowEvent ev)
   {
   }   
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
   **/
   public void windowIconified(java.awt.event.WindowEvent ev)
   {
   }   
   /**
	* this method is a part of the Windowlistener interface used
	* to listen to window events.
   **/
   public void windowOpened(java.awt.event.WindowEvent ev)
   {
   }   
}