
package macrolanguage;

import sun.tools.javac.Main;
import java.util.*;
import java.lang.*;
import java.io.*;
import java.util.zip.*;
import java.util.zip.ZipInputStream;
import macrolanguage.SimpleStream;
import java.net.URL;

/**
 * To use the MacroExtention you must simply add it to your program
 * and add an interface for selecting the macro, then use the simple
 * methods of this class to compile and execute a macro.
 * The class offers several features:
 * 1. Interation with your application.
 * 2. The full power you had writing the application will be available
 *    to your users.
 * 3. You can add a sandbox to block some functionality.
 * 4. You can ship upgrades/fixes as plugins.
 * 5. Other vendors and users can easly write plugins.
 * 6. Plugins will be fully integrate, cross platform and run in the 
 *    same speed as the VM.
 * It is recommended you expose some of your softwares internal's
 * to your users to make this tool the most effective.
 *
 * There is great room for improvment in the loading from JAR's.
 * did not implemtent any cahcing and the loader searchs the JAR every
 * time. It should be rather simple to implement a Hashtable which will 
 * cache the JAR.
**/
public class MacroExtention extends ClassLoader
{
	/**
	 * Notice the long name. It's here to avoid namespace 
	 * problems with another Main
	**/ 
	private	sun.tools.javac.Main compilerClass;  
	private SimpleStream dataStream = new SimpleStream();

	// a caching for the class files already loaded.
 	private Hashtable classCache = new Hashtable();
 	
 	// the JAR file currently used. null if the classes
 	// are searched in the current directory.
 	private transient String jarFileName = null;
 	
 	// the size of the current zip stream.
 	private transient int streamSize;

	/**
	 * the default constructor does nothing except create an instance 
	 * of sun.tools.javac.Main for later use.
	**/
	MacroExtention()
	{
		compilerClass = new sun.tools.javac.Main(dataStream,"");
	}
	/**
	 * This method accepts a java file as parameter
	 * and compiles it.
	 * It returns true if compilation was successful
	 * and false otherwize.
	**/
	public boolean compile(String fileName)
	{
		String [] stringArr = new String[1];
		stringArr[0] = fileName;
		return(compilerClass.compile(stringArr));
	}
/**
 * This method returns the data within the disk file to the class loader 
 * so it can load the class. 
 * This version of the method loads the class file from the  disk.
 * The className is the name and path of the file.
**/
public byte[] getClassData(String className ) 
	throws IOException
{
	File classFile;
	String classFullName;
	
	// if the class file ends with the .class extention then remove it.	
	if (!className.endsWith(".class"))
 		classFullName = className + ".class";
	else
	{
		className = className.substring(0, className.length() - 5);
		System.out.println("Class name is: " + className);
 		classFullName = className + ".class";
	}

	// open the filename containing the class.
	classFile = new File(classFullName);

	// get the size of the file so we can know the amount of memory
	// to allocate for it.	
	int sizeOfFile = (int)classFile.length();

	// read the class into memory.
	InputStream inputStream = new FileInputStream(classFile);

	byte[] returnValue = new byte[sizeOfFile];
	
	inputStream.read(returnValue);
	
	inputStream.close();

	// return the data within the class.
	return(returnValue);
}
/**
 * This method returns the data within the JAR to the class loader so it 
 * can load a class. 
 * This version of the method loads the class file from a JAR
 * archive. The className is the name and path of the file in the JAR
 * the jarFileName is the full name of the JAR file.
**/
public byte[] getClassDataFromJAR(String className, String jarFileName ) 
	throws IOException
{
	File jarFile;
	String classFullName;

	// remove the .class from the end of the name.
	if (!className.endsWith(".class"))
		className = className;
	else
		className = className.substring(0, className.length() - 5);

	classFullName = className;
	
	// the classes are held in the zip files as sub directories.
	// we should replace the dot notation with the slash notation
	// to reach the correct path name.
	classFullName = classFullName.replace('.','/');

	// add the .class to the end of the full name.
	classFullName += ".class";

	try
	{
		// get a stream from a zip (JAR) file.
		InputStream zipStream = getJarInputStream (classFullName, jarFileName);  
	
		// allocate the correct amount of memory for the file. If the size
		// of memory needed is smaller than 0 that means the size is not
		// stored or corrupt.
		byte[] returnValue;	
		if(streamSize < 0)
		{
			returnValue = new byte[1000];
			zipStream.read(returnValue, 0, 1000);
		}	
		else
		{
			returnValue = new byte[streamSize];
			zipStream.read(returnValue, 0, streamSize);
		}	
 
		return(returnValue);
	}	
	catch(Exception err)
	{
		// if the class is not found in the ZIP try finding it in
		// the normal way useing the getClassData method.
		return(getClassData(className));
	}	
}
	/**
	 * This method returns a vector in which all 
	 * the compilation errors are stored as strings.
	**/
	public Vector getCompilationErrors()
	{
		return(dataStream.getStringsWritten());
	}
/**
 * this method returns an InputStream to the fileName we are looking
 * for. The fileName must be fully qualified for search in the zip:
 * all subdirectories shiould use the / notation, i.e.:
 * this file \VisualDeveloper\FreeFormEdit.class should in fact be
 * /VisualDeveloper/FreeFormEdit.class
**/
private synchronized InputStream getJarInputStream (String fileName, String jarFileName) 
	throws IOException
{
	// file object to the jar file.
	File jarFile = new File(jarFileName);

	// open a zip input stream.
	ZipInputStream zipStream = new ZipInputStream(new FileInputStream(jarFile));

	// iterate through the entries until we find on that equals
	// the fileName we are looking for.
	ZipEntry currentEntry = zipStream.getNextEntry();
	while (!currentEntry.getName().equals(fileName))
	{
		zipStream.closeEntry();			
		currentEntry = zipStream.getNextEntry();
	}

	streamSize = (int)currentEntry.getSize();
	
	return(zipStream);
}
	/**
	 * This method accepts a class file as parameter
	 * and returns an instance of it.
	**/
	public Object getObject(String fileName) 
		throws	java.lang.ClassNotFoundException,
				java.lang.InstantiationException,
				java.lang.IllegalAccessException
	{
		Class classInstance = getClass();
		return(classInstance.forName(fileName).newInstance());
	}
/**
 * Find a resource with a given name. The return is a URL to the resource. 
 * Doing a getContent() on the URL may return an Image, an AudioClip, or 
 * an InputStream.
 * The class loader can choose what to do to locate the resource. 
**/
public URL getResource(String name) 
{
	// try to locate the resource useing the default class loader.
	URL returnValue = getSystemResource(name);

	// if the class loader was successful then return the result.
	if (returnValue != null) 
		return returnValue;
	
	Object obj = null;

	if (obj != null) {
		// Create a URL to refer to this resource
		try 
		{
			URL url = new URL("",null, "/"+ "/+/"+name);
			return url;
		} 
		catch (Exception e) 
		{
			return null;
		}
	}
	return null;
}
/**
 * Get an InputStream on a given resource. Will return null if no 
 * resource with this name is found. 
 * The class loader can choose what to do to locate the resource. 
**/
public InputStream getResourceAsStream(String name) {
	InputStream returnValue = getSystemResourceAsStream(name);
	if (returnValue != null) 
		return returnValue;
	
	try
	{
		// return the file stream.
		return (getJarInputStream (name, jarFileName)); 	
	}
	catch (IOException err)
	{
		// an io error ocurred retuen null
		return(null);
	}	
}
	/**
	 * the load class method loads a class file from the disk or from a
	 * jar file and resolves the class with the JVM.
	 * This method can be extended to support loading classes from a web
	 * site or any othe source.
	 * When developing a class loader security can be a major thorn,
	 * if you load classes from the internet make sure you habdle the
	 * security issue correctly.
	**/
	protected Class loadClass(String className, boolean callReslove)
 	{
 		// check if the class was already loaded.
		Class returnValue = (Class) classCache.get(className);

		// if the class was already loaded return it.
  		if(returnValue != null) return(returnValue);

 		try
	   {	
	   		// this variable will hold all the data in the .class file.
	   		byte[] classData;
	   		
	   		// check whether we are useing a JAR file name.
	   		if(jarFileName != null)
	   			// load the class file from the JAR file.
	   			classData = getClassDataFromJAR(className, jarFileName);
	   		else
				// alocate enough memory to load the file into memory.
 		    	classData = getClassData(className);

			returnValue = defineClass(className, classData, 0, classData.length);

			classCache.put(className, returnValue);
		}
  		catch(IOException ioerr)
		{
			try 
		 {
			returnValue = findSystemClass(className);
			return(returnValue);
		 }
		 catch (Exception anyException)
		 {
	        	System.out.println("File access error: " + anyException.getMessage());
 	      	return(null);
		 }
	  }

		if(callReslove)
		{  	
  			resolveClass(returnValue);
  		}

		return(returnValue);
   }                              
	/**
	 * creats an instance of a class by getting its name
	 * and returns that instance, or null if none were found.
	**/
	public Object loadObject(String className)
	{
		try 
	  	{
	  		// load the class into memory, and then get an instance of 
	  		// an object of the class and return it.
	  		Class classReference = loadClass(className, true);
		 	return(classReference.newInstance());
	  	}
		catch(java.lang.InstantiationException e)
  		{
			System.out.println("Instantiation exception: " + e.getMessage());
	  	}
		catch(java.lang.IllegalAccessException e)
  		{
			System.out.println("Illegal access exception : " + e.getMessage());
	  	}
	  	catch(Exception e)   	
		{
			System.out.println("An exception was thrown: " + e.getMessage());
		}
		return(null);
	}
	/**
	 * I use this method in my experimenting and chose not to remove it
	 * since it may be of interest for the readers.
	**/
	public static void main(String argv[])
 	{
 		if (argv.length == 0)
 		{
 			System.out.println("you must enter one argument: the file name to compile and run (no extention). and possibly a JAR file name.");
 			return;
 		}	
		MacroExtention m = new MacroExtention();
		if(argv.length == 2)
		{
			m.setJARFileName(argv[1]);
		}	
		m.loadObject(argv[0]);
   }               
/**
 * this is a setter for the current JAR file name.
 * once set to a certin file all class loader queries will
 * be redirected to the JAR fileName.
 * This must be set to null when no JAR is being used.
**/
public void setJARFileName(String fileName) {
	jarFileName = fileName;
}
}